/* src/cs_register.c - Enregistrement de salon
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: cs_register.c,v 1.58 2005/09/01 16:47:59 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "add_info.h"
#include "fichiers.h"
#include "outils.h"
#include "cs_cmds.h"
#include "config.h"
#ifdef SQLLOG
#include "sql_log.h"
#endif

/*
 * register_chan <salon> <description>
 */
int register_chan(aNick *nick, aChan *c, int parc, char **parv)
{
	char *salon = parv[1], theme[DESCRIPTIONLEN + 1] = {0}, *ptr;
	aJoin *j;

	if(!chan_check(salon, nick)) return 0;

	if(CantRegChan(nick->user))
		return csntc(nick, "Vous ne pouvez pas enregistrer de salon%s%s.",
				(nick->user->cantregchan > 0) ? " jusqu'au " : "",
				(nick->user->cantregchan > 0) ? get_time(nick, nick->user->cantregchan) : "");

	parv2msgn(parc, parv, 2, theme, DESCRIPTIONLEN);

	if(strlen(theme) < 12)
		return csntc(nick, GetReply(nick, L_MINDESCRIPTIONLEN));

	if((ptr = IsAnOwner(nick->user)))
		return csntc(nick, GetReply(nick, L_ALREADYOWNER), ptr);

	if(getchaninfo(salon)) return csntc(nick, GetReply(nick, L_ALREADYREG), salon);

	if(!(j = getjoininfo(nick, salon)) || !IsOp(j))
		return csntc(nick, GetReply(nick, L_NEEDTOBEOP), salon);

	c = add_chan(salon, theme);
	add_access(nick->user, salon, OWNERLEVEL, A_OP | A_PROTECT, CurrentTS);

	csjoin(c, JOIN_REG|JOIN_FORCE);
	cstopic(c, theme);
#ifndef SQLLOG
	putlog(LOG_CHANS, "REGCHAN %s par %s@%s (%s)", salon, nick->nick, nick->user->nick, theme);
#endif
	csntc(nick, GetReply(nick, L_CHANNOWREG), salon);
	return 1;
}

/*
 * ren_chan <chan> <nouveau chan>
 */
int ren_chan(aNick *nick, aChan *c, int parc, char **parv)
{
	aJoin *j;
	const char *newchan = parv[2];

	if(c->flag & C_ALREADYRENAME && !IsAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_CMDALREADYUSED), parv[0]);

	if(!chan_check(newchan, nick)) return 0;

	if(getchaninfo(newchan))
		return csntc(nick, GetReply(nick, L_ALREADYREG), newchan);

	if(!IsAdmin(nick->user) && (!(j = getjoininfo(nick, newchan)) || !IsOp(j)))
		return csntc(nick, GetReply(nick, L_NEEDTOBEOP), newchan);

#ifdef SQLLOG
	sql_query(SQL_QINSERTC, "('%s', '%s', %T, 'renchan', 'Old name: %s')", nick->user->nick,
 		newchan, CurrentTS, c->chan);
#else
	putlog(LOG_CHANS, "RENCHAN %s %s par %s@%s", c->chan, newchan, nick->nick, nick->user->nick);
#endif
	if(CJoined(c)) cspart(c, newchan);
	if(c->netchan) c->netchan->regchan = NULL, c->netchan = NULL; /* csjoin will find the new NetChan */
	switch_chan(c, newchan);
	csjoin(c, JOIN_FORCE);
	c->flag |= C_ALREADYRENAME;

	csntc(nick, GetReply(nick, L_OKCHANGED));
	return 1;
}

/*
 * unreg_chan salon raison
 */
int unreg_chan(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char tmp[300];

	if(!chaninfo->owner || chaninfo->owner->user != nick->user)
		return csntc(nick, GetReply(nick, L_NEEDTOBEOWNER));

	if(parc < 2 || strcasecmp(parv[2], "CONFIRME"))
		return csntc(nick, GetReply(nick, L_NEEDCONFIRM));

	csntc(nick, GetReply(nick, L_CHANUNREG), parv[1]);
	snprintf(tmp, sizeof tmp, "Unregister par %s@%s (%s)", nick->nick, nick->user->nick,
		parc > 2 ? parv2msg(parc, parv, 3, 150) : "Aucune raison");

	if(cf_unreg_reg_delay) nick->user->cantregchan = CurrentTS + cf_unreg_reg_delay;
	del_chan(chaninfo, 0, tmp);
	return 1;
}
