/* src/vote.c - Effectuer un vote
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: vote.c,v 1.28 2005/08/23 19:57:42 romexzf Exp $
 */

#include "main.h"
#ifdef HAVE_VOTE
#include "vote.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "hash.h"
#include "outils.h"

struct votes vote[MAXVOTEPROP + 1];
struct Vote Vote = {0, 0, 0};

static int vote_sort(const void *a, const void *b)
{
	const struct votes *va = a;
	const struct votes *vb = b;
	return vb->voies - va->voies;
}

static void show_results(aNick *nick, struct votes *show)
{
	int i = 1;

	csntc(nick, "Resultats du vote\2 %s\2", show[0].prop);

	for(;i <= Vote.nbprop;++i)
		csntc(nick, "%d. %.2f%% [%d] \2%s", i, (show[i].voies * 100.0) / Vote.votes,
			show[i].voies, show[i].prop);
}

void show_vote(aNick *nick)
{
	int i = 1;
	csntc(nick, "\2Sujet:\2 %s", vote[0].prop);
	csntc(nick, "\2Propositions:\2");
	for(;i <= Vote.nbprop;++i) csntc(nick, "%d. %s", i, vote[i].prop);
}

/*
 * vote_results
 */
int vote_results(aNick *nick, aChan *c, int parc, char **parv)
{
	if(Vote.actif)
		return csntc(nick, "Vous ne pouvez pas voir les rsultats, le vote est encore en cours.");

	if(!Vote.nbprop) return csntc(nick, "Il n'y a aucun vote existant.");

	if(nick->user->level < Vote.level)
		return csntc(nick, GetReply(nick, L_NEEDTOBEADMIN));

	show_results(nick, vote);

	return 1;
}

/*
 * voter parv[1] = n de la proposition
 */
int voter(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	int nb_prop = 0;

	if(nick->user->level < Vote.level)
		return csntc(nick, GetReply(nick, L_NEEDTOBEADMIN));

	if(!Vote.actif)
	{
		if(!Vote.nbprop) return csntc(nick, "Le vote n'est pas actif");
		else return csntc(nick, "Le vote est clos. Pour voir les rsultats, tapez /%s %s",
						cs.nick, RealCmd("results"));
	}

	if(!parc)
	{
		csntc(nick, "Un vote est actuellement en cours.");
		show_vote(nick);
		if(nick->user && !UVote(nick->user))
			csntc(nick, "Pour voter utilisez: \2/%s %s <n de la proposition>\2", cs.nick, parv[0]);
		return 1;
	}

	if(nick->user->reg_time > Vote.start_time)
		return csntc(nick, "Vous avez enregistr votre username aprs l'ouverture du vote."
			"Vous ne pourrez voter qu'au prochain vote");

	if(UVote(nick->user)) return csntc(nick, "Vous avez dj vot.");

	if(!is_num(parv[1])) return csntc(nick, "Le n de la proposition doit tre un nombre.");

	if((nb_prop = atoi(parv[1])) < 1 || nb_prop > Vote.nbprop)
		return csntc(nick, "Les propositions vont de 1  %d", Vote.nbprop);

	SetUVote(nick->user);
	++vote[nb_prop].voies;
	++Vote.votes;
	csntc(nick, "Votre vote pour \2%s\2 a bien t pris en compte.", vote[nb_prop].prop);
	write_votes(); /* to avoid multiple votes in case of crash */
	return 1;
}

/*
 * vote NEW <nombre de prop> <level> <sujet>
 * 		PROPOSITION <proposition>
 * 		CLORE
 * 		RESULTS
 */

int do_vote(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];

	if(!strcasecmp(cmd, "NEW"))
	{
		anUser *u;
		char *msg = NULL;
		int i = 0;

		if(parc < 4)
			return csntc(nick, "Syntaxe: %s NEW <nombre de propositions> <level> :<sujet>", parv[0]);

		if(Vote.actif) return csntc(nick, "Le vote est dj actif");

		if(!is_num(parv[2]))
			return csntc(nick, "Le nombre de propositions doit tre une valeur numrique");

		if((Vote.nbprop = atoi(parv[2])) < 2 || Vote.nbprop > MAXVOTEPROP)
			return csntc(nick, "Il peut y avoir de 2  %d propositions possibles.", MAXVOTEPROP);

		if(!is_num(parv[3]) || (Vote.level = atoi(parv[3])) > MAXADMLVL || Vote.level < 1)
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		msg = parv2msg(parc, parv, 4, MEMOLEN + 3);
		if(strlen(msg) > MEMOLEN)
			return csntc(nick, "La longueur du sujet du vote est limite  %d caractres", MEMOLEN);

		memset(vote, 0, sizeof vote);
		Strncpy(vote[0].prop, msg, MEMOLEN);
		for(;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = u->next) DelUVote(u);

		csntc(nick, "Le vote est en cours de cration au level %d.", Vote.level);
		csntc(nick, "\2Sujet:\2 %s", vote[0].prop);
		csntc(nick, "Vous devez maintenant dfinir les propositions avec: "
				"\2/%s %s PROPOSITION :<nom>\2 jusqu' que vous ayez donn les %d propositions",
				cs.nick, parv[0], Vote.nbprop);
		Vote.votes = 0;
		Vote.start_time = CurrentTS;
	}
	else if(!strcasecmp(cmd, "PROPOSITION"))
	{
		int i = 1;
		char *msg = NULL;

		if(parc < 2) return csntc(nick, "Syntaxe: %s PROPOSITION :<proposition>", parv[0]);

		if(Vote.actif) return csntc(nick, "Le vote est dj actif");

		if(!Vote.nbprop) return csntc(nick, "Aucun vote n'est en cours d'enregistrement");

		for(;*vote[i].prop && i <= Vote.nbprop;++i);
		if(i > Vote.nbprop)
			return csntc(nick, "Vous avez bien donn toutes les propositions pour le vote");

		msg = parv2msg(parc, parv, 2, MEMOLEN + 3);
		if(strlen(msg) > MEMOLEN)
			return csntc(nick, "La longueur d'une proposition est limite  %d caractres", MEMOLEN);

		Strncpy(vote[i].prop, msg, MEMOLEN);
		vote[i].voies = 0;

		if(i == Vote.nbprop)
		{
			Vote.actif = 1;
			csntc(nick, "Le vote est maintenant \2ACTIF\2.");
			show_vote(nick);
			write_votes();
		}
		else csntc(nick, "Vous avez donn %d des %d propositions.", i, Vote.nbprop);
	}
	else if(!strcasecmp(cmd, "CLORE"))
	{
		if(!Vote.actif) return csntc(nick, "Il n'y a pas de vote actif.");

		Vote.actif = 0;
		qsort(vote + 1, Vote.nbprop, sizeof *vote, vote_sort);
		csntc(nick, "Le vote est clos, les users peuvent maintenant voir les rsultats.");
		write_votes();
	}
	else if(!strcasecmp(cmd, "RESULTS"))
	{
		struct votes lvote[MAXVOTEPROP + 1];
		int i = 0, usr = 0;
		anUser *u;

		if(!Vote.nbprop) return csntc(nick, "Il n'y a aucun vote existant.");

		if(Vote.actif)
		{
			memcpy(lvote, vote, sizeof vote);
			qsort(lvote + 1, Vote.nbprop, sizeof *lvote, vote_sort);
		}
		show_results(nick, Vote.actif ? lvote : vote);

		for(;i < USERHASHSIZE;++i) for(u = user_tab[i];u;u = u->next)
			if(u->level >= Vote.level && u->reg_time <= Vote.start_time) ++usr;

		csntc(nick, "%d des %d users ont vot (%.2f%%)", Vote.votes, usr, (Vote.votes*100.0)/ usr);
	}
	else csntc(nick, "La sous commande %s n'existe pas", cmd);
	return 1;
}

void write_votes(void)
{
   FILE *fp = fopen(VOTE_FILE, "w");
   int i = 1;

   if(!fp) return;

   fprintf(fp, "VOTE %d %ld %d :%s\n", Vote.actif, Vote.start_time, Vote.level, vote[0].prop);
   for(;i <= Vote.nbprop;++i) fprintf(fp, "PROP %d :%s\n", vote[i].voies, vote[i].prop);
   fclose(fp);
}

void load_votes(void)
{
   FILE *fp = fopen(VOTE_FILE, "r");
   char buf[300], *ar[5];

   if(!fp) return;
   while(fgets(buf, sizeof buf, fp))
   {
	   int items = split_buf(buf, ar, ASIZE(ar));
	   strip_newline(ar[items - 1]);
	   if(!strcmp(buf, "VOTE") && items > 4)
	   {
		   Vote.actif = strtol(ar[1], NULL, 10);
		   Vote.start_time = strtol(ar[2], NULL, 10);
		   Vote.level = strtol(ar[3], NULL, 10);
		   Strncpy(vote[0].prop, ar[4], MEMOLEN);
	   }
	   else if(!strcmp(buf, "PROP") && items > 2)
	   {
		   Strncpy(vote[++Vote.nbprop].prop, ar[2], MEMOLEN);
		   vote[Vote.nbprop].voies = strtol(ar[1], NULL, 10);
		   Vote.votes += vote[Vote.nbprop].voies;
	   }
   }
   fclose(fp);
}

#endif
